# hist.py
# Calculates a histogram of a set of strings in a file with
# one per line.  Filename is the single argument.
# Author: Neil C. Rowe, 3/14
import sys

# Allows same program to handle Windows and Linux line reads
def endclean(OS):
    S = OS
    M = len(S)
    while ((M > 0) and \
           (S[M-1] in ['\n','\r','\t','\f','\a','\b','\v',' ']) ):     
        S = S[0:M-1]
        M = M-1
    return S   

def fixedlengthstring(intnum,numdigits):
    sintnum = str(intnum)
    M = len(sintnum)
    for i in list(range(M+1,numdigits+1)):
        sintnum = '0' + sintnum
    return sintnum

def hist(infilename):
    k = max(infilename.rfind('/'),infilename.rfind('\\'))
    if (k > -1):
        dirname = infilename[0:k+1]
        justfilename = infilename[k+1:len(infilename)]
    else:
        dirname = ''
        justfilename = infilename
    print('Computing histogram on',infilename)
    hashtable = {}
    totalcount = 0
    with open(infilename, 'r', encoding='utf=8') as f:
        for val in f:
            val = endclean(val)
            count = hashtable.get(val,0)
            hashtable[val] = count + 1
            totalcount = totalcount + 1
    countmap = []
    valuelist = []
    namelist = []
    numvalues = 0
    for key in hashtable.keys():
        count = hashtable[key]
        countmap.append(list([count,numvalues]))
        valuelist.append(key)
        namelist.append(key + ': ' + str(count))
        numvalues = numvalues + 1
    countmap.sort()
    outfile1 = open(dirname + 'hist_count_sort_' + justfilename, 'w', \
                    encoding='utf-8')
    N = len(countmap)
    for i in list(range(N)):
        outfile1.write(str(countmap[N-i-1][0]) + ' ' + \
                       valuelist[countmap[N-i-1][1]] + '\n')
    outfile1.close()
    outfile2 = open(dirname + 'hist_name_sort_' + justfilename, 'w', \
                    encoding='utf-8')
    namelist.sort()
    for i in list(range(N)):
        outfile2.write(namelist[i] + '\n')
    outfile2.close()

if __name__=="__main__":
    infilename = sys.argv[1]
    hist(infilename)
