# join_files.py
# Matches given columns of two files, outputs the result; ignores
# lines that do not match.  If multiple matches in first file, each
# generates a separate line of output.
# Caches lines of first file, which should be smaller than the second file.
# Output file has word "join" in front of first input filename.
# Args. are file 1, file 2, column 1, column 2.
# Author: Neil C. Rowe, 10/14
import sys

# Allows same program to handle Windows and Linux line reads
def endclean(OS):
    S = OS
    M = len(S)
    while ((M > 0) and \
           (S[M-1] in ['\n','\r','\t','\f','\a','\b','\v',' ']) ):          
        S = S[0:M-1]
        M = M-1
    return S

def join_files(file1,file2,col1,col2,delimchar):
    print('Joining file',file1,'column',col1,'with',file2)
    print('column',col2,'using delimiter',delimchar)
    ks = file1.rfind('/')
    if (ks > -1):
        outfilename = file1[0:ks+1] + 'join_' + file1[ks+1:len(file1)]
    else:
        outfilename = 'join_' + file1
    # Create hash table of all rows of first file matching a particular
    # value in col1
    hashtable = {}
    linenum = 0
    with open(file1, 'r', encoding='utf-8') as f:
        for line in f:
            line = endclean(line)
            pline = line.split(delimchar)
            if (len(pline) > col1):
                hashval = hashtable.get(pline[col1],[])
                hashval.append(line)
                hashtable[pline[col1]] = hashval
            else:
                print('Line',linenum,'of',file1,'is too short')
            linenum = linenum + 1
    print(linenum,'entries hashed from',file1)
    print('Output file will be:',outfilename)
    linenum = 0
    out = open(outfilename, 'w', encoding='utf-8')
    with open(file2, 'r', encoding='utf-8') as f:
        for line in f:
            line = endclean(line)
            pline = line.split(delimchar)
            if (len(pline) > col2):
                hashval = hashtable.get(pline[col2],[])
                for i in list(range(len(hashval))):
                    out.write(hashval[i] + delimchar + line + '\n')
            else:
                print('Line',linenum,'of',file2,'is too short')
            linenum = linenum + 1
    out.close()

if __name__=="__main__":
    file1 = sys.argv[1]
    file2 = sys.argv[2]
    col1 = int(sys.argv[3])
    col2 = int(sys.argv[4])
    delimchar = sys.argv[5]
    join_files(file1,file2,col1,col2,delimchar)

